extern int debug;
extern int do_compress;
extern int fhs;
extern int fsstnd;
extern int noautopath;
extern int nocache;
extern int findall;
extern int nocats;
extern int preformat;
extern int do_troff;
extern int apropos;
extern int whatis;
extern int print_where;
extern int one_per_line;
extern int do_irix;
extern char *dohp;
extern const char *progname;
extern const char *pager;
extern const char *browser;
extern const char *htmlpager;
extern char *colon_sep_section_list;
extern char *roff_directive;
