BAD_CONFIG_FILE
	"nie można zrozumieć zawartości pliku %s\n"
CONFIG_OPEN_ERROR
	"Ostrzeżenie: nie można otworzyć pliku konfiguracyjnego %s\n"
PARSE_ERROR_IN_CONFIG
	"Błąd podczas interpretacji pliku konfiguracyjnego\n"
INCOMPAT
	"niepasujące wzajemnie opcje %s i %s\n"
NO_ALTERNATE
	"Niestety - brak wkompilowanej obsługi alternatywnych systemów\n"
NO_COMPRESS
	"Man został skompilowany z automatyczną kompresją stron cat,\n\
ale plik konfiguracyjny nie definiuje COMPRESS.\n"
NO_NAME_FROM_SECTION
	"Którą stronę podręcznika z sekcji %s chcesz przeczytać?\n"
NO_NAME_NO_SECTION
	"Którą stronę podręcznika chcesz przeczytać?\n"
NO_SUCH_ENTRY_IN_SECTION
	"Nie ma strony %s w sekcji %s podręcznika\n"
NO_SUCH_ENTRY
	"Nie ma strony podręcznika dla %s\n"
PAGER_IS
	"\nużywanie %s jako pagera\n"
SYSTEM_FAILED
	"Błąd podczas wykonywania polecenia formatowania lub wyświetlania.\n\
Polecenie systemowe %s zwróciło status %d.\n"
VERSION
	"%s, wersja %s\n\n"
OUT_OF_MEMORY
	"Brak pamięci - nie można przydzielić %d bajtów pamięci\n"
ROFF_CMD_FROM_FILE_ERROR
	"Błąd podczas przetwarzania polecenia *roff z pliku %s\n"
MANROFFSEQ_ERROR
	"Błąd podczas przetwarzania MANROFFSEQ. Używanie ustawień domyślnych systemu.\n"
ROFF_CMD_FROM_COMMANDLINE_ERROR
	"Błąd podczas przetwarzania polecenia *roff podanego w linii poleceń.\n"
UNRECOGNIZED_LINE
	"Nierozpoznana linia w pliku konfiguracyjnym (zignorowana)\n%s\n"
GETVAL_ERROR
	"man-config.c: błąd wewnętrzny: nie znaleziono łańcucha znaków %s\n"
FOUND_MANDIR
	"znaleziono katalog man %s\n"
FOUND_MAP
	"znaleziono mapę manpath %s --> %s\n"
FOUND_CATDIR
	"odpowiadający katalog cat to %s\n"
LINE_TOO_LONG
	"Za długa linia w pliku konfiguracyjnym\n"
SECTION
	"\nsekcja: %s\n"
UNLINKED
	"skasowany %s\n"
GLOBBING
	"globbing %s\n"
EXPANSION_FAILED
	"Próba [%s] rozwinięcia strony podręcznika nie powiodła się\n"
OPEN_ERROR
	"Nie można otworzyć strony podręcznika %s\n"
READ_ERROR
	"Błąd podczas odczytu strony podręcznika %s\n"
FOUND_EQN
	"znaleziono dyrektywę eqn(1)\n"
FOUND_GRAP
	"znaleziono dyrektywę grap(1)\n"
FOUND_PIC
	"znaleziono dyrektywę pic(1)\n"
FOUND_TBL
	"znaleziono dyrektywę tbl(1)\n"
FOUND_VGRIND
	"znaleziono dyrektywę vgrind(1)\n"
FOUND_REFER
	"znaleziono dyrektywę refer(1)\n"
ROFF_FROM_COMMAND_LINE
	"przetwarzanie dyrektywy podanej w linii poleceń\n"
ROFF_FROM_FILE
	"przetwarzanie dyrektywy z pliku %s\n"
ROFF_FROM_ENV
	"przetwarzanie dyrektywy ze zmiennej środowiskowej\n"
USING_DEFAULT
	"używanie domyślnej sekwencji preprocesora\n"
PLEASE_WAIT
	"Formatowanie strony, proszę czekać...\n"
CHANGED_MODE
	"zmieniono uprawnienia %s na %o\n"
CAT_OPEN_ERROR
	"Nie można otworzyć %s do zapisu.\n"
PROPOSED_CATFILE
	"w razie potrzeby man będzie próbował zapisać %s\n"
IS_NEWER_RESULT
	"status z is_newer() = %d\n"
TRYING_SECTION
	"próbowanie sekcji %s\n"
SEARCHING
	"\nwyszukiwanie w %s\n"
ALREADY_IN_MANPATH
	"ale %s jest już w manpath\n"
CANNOT_STAT
	"Ostrzeżenie: nie można użyć stat na pliku %s!\n"
IS_NO_DIR
	"Ostrzeżenie: %s nie jest katalogiem!\n"
ADDING_TO_MANPATH
	"dodawanie %s do manpath\n"
PATH_DIR
	"\nścieżka katalogu %s "
IS_IN_CONFIG
	"jest w pliku konfiguracyjnym\n"
IS_NOT_IN_CONFIG
	"nie jest obecna w pliku konfiguracyjnym\n"
MAN_NEARBY
	"ale jest katalog man niedaleko\n"
NO_MAN_NEARBY
	"i nie ma katalogu man niedaleko\n"
ADDING_MANDIRS
	"\ndodawanie obowiązkowych katalogów man\n\n"
CATNAME_IS
	"cat_name w convert_to_cat () jest: %s\n"
NO_EXEC
	"\nniewykonywanie polecenia:\n  %s\n"
USAGE1
	"użycie: %s [-adfhktwW] [sekcja] [-M ścieżka] [-P pager] [-S lista]\n\t"
USAGE2
	"[-m system] "
USAGE3
	"[-p łańcuch_znaków] nazwa ...\n\n"
USAGE4
	"  a : znajduje wszystkie pasujące strony\n\
  c : nie używa pliku cat\n\
  d : wyświetla mnóstwo informacji debugowania\n\
  D : jak -d, ale wyświetla też strony\n\
  f : to samo co whatis(1)\n\
  h : wyświetla ten komunikat pomocy\n\
  k : to samo co apropos(1)\n\
  K : wyszukuje łańcuch znaków na wszystkich stronach\n"
USAGE5
	"  t : używa troff do formatowania stron do wydrukowania\n"
USAGE6
	"\
  w : wyświetla położenie stron(y) podręcznika, która byłaby wyświetlona\n\
      (jeżeli nie podano żadnej nazwy: wyświetla przeszukiwane katalogi)\n\
  W : tak jak -w, ale wyświetla tylko nazwy plików\n\n\
  C plik   : używa `plik' jako plik konfiguracyjny\n\
  M ścieżka: ustawia ścieżkę wyszukiwania stron podręcznika jako `ścieżka'\n\
  P pager  : używa programu `pager' do wyświetlania stron\n\
  S lista  : lista sekcji oddzielona dwukropkami\n"
USAGE7
	"  m system : wyszukuje strony podręcznika dla alternatywnego systemu\n"
USAGE8
	"  p łańcuch znaków: łańcuch znaków określający, które preprocesory użyć\n\
               e - [n]eqn(1)   p - pic(1)    t - tbl(1)\n\
               g - grap(1)     r - refer(1)  v - vgrind(1)\n"
USER_CANNOT_OPEN_CAT
	"a rzeczywisty użytkownik też nie może otworzyć pliku\n"
USER_CAN_OPEN_CAT
	"ale rzeczywisty użytkownik może otworzyć plik\n"
CANNOT_FORK
	"nieudana próba fork polecenia _%s_\n"
WAIT_FAILED
	"błąd podczas oczekiwania na proces potomny _%s_\n"
GOT_WRONG_PID
	"bardzo dziwne..., otrzymano zły pid podczas oczekiwania na proces potomny\n"
CHILD_TERMINATED_ABNORMALLY
	"błąd krytyczny: polecenie _%s_ zakończone nieprawidłowo\n"
IDENTICAL
	"Strona podręcznika %s jest identyczna jak %s\n"
MAN_FOUND
	"Znaleziono stronę(y) podręcznika:\n"
NO_TROFF
	"błąd: nie podano polecenia TROFF w %s\n"
NO_CAT_FOR_NONSTD_LL
	"strona cat nie została zapisana z powodu niestandardowej długości linii\n"
BROWSER_IS
	"\nużywanie %s jako przeglądarki\n"
HTMLPAGER_IS
	"\nużywanie %s do zrzucania stron HTML jako tekst"
FOUND_FILE
	"manfile_from_sec_and_dir() znalazła %s\n"
CALLTRACE1
	"manfile_from_sec_and_dir(dir=%s, sec=%s, name=%s, flags=0x%0x)\n"
CALLTRACE2
	"glob_for_file(dir=%s, sec=%s, name=%s, type=0x%0x, ...)\n"
NO_MATCH
	"glob_for_file nie znalazła żadnych dopasowań.\n"
GLOB_FOR_FILE
	"glob_for_file zwróciła %s.\n"
CALLTRACE3
	"glob_for_file_ext_glob(dir=%s, sec=%s, name=%s, ext=%s, hpx=%s, glob=%d, type=0x%0x);\n"
ABOUT_TO_GLOB
	"glob_for_file_ext_glob rozwinie %s\n"
