BAD_CONFIG_FILE
	"αδύνατον να βγάλω νόημα από το αρχείο %s\n"
CONFIG_OPEN_ERROR
	"Προειδοποίηση: αδύνατον ν' ανοίξω το αρχείο διευθέτησης %s\n"
PARSE_ERROR_IN_CONFIG
	"Σφάλμα κατά την ανάλυση του αρχείου διευθέτησης\n"
INCOMPAT
	"οι επιλογές %s και %s είναι ασύμβατες μεταξύ τους\n"
NO_ALTERNATE
	"Λυπούμαι - δεν παρέχεται υποστήριξη για εναλλακτικό σύστημα σελίδων\n"
NO_COMPRESS
	"Το πρόγραμμα man έχει μεταγλωττιστεί ώστε να κάνει αυτόματη συμπίεση,\n\
αλλά το αρχείο διευθέτησης δεν προσδιορίζει το πρόγραμμα\n\
συμπίεσης COMPRESS.\n"
NO_NAME_FROM_SECTION
	"Ποιά σελίδα εγχειριδίου θέλετε από την ενότητα %s;\n"
NO_NAME_NO_SECTION
	"Ποιά σελίδα εγχειριδίου θέλετε;\n"
NO_SUCH_ENTRY_IN_SECTION
	"Δεν υπάρχει λήμμα για %s στην ενότητα %s του εγχειριδίου\n"
NO_SUCH_ENTRY
	"Δεν υπάρχει σελίδα εγχειριδίου για %s\n"
PAGER_IS
	"\nχρήση του %s ως σελιδοποιού\n"
SYSTEM_FAILED
	"Σφάλμα κατά τη μορφοποίηση ή κατά την εμφάνιση.\n\
Η εντολή συστήματος %s επιστρέφει το σφάλμα %d.\n"
VERSION
	"%s, έκδοση %s\n\n"
OUT_OF_MEMORY
	"Όχι αρκετή μνήμη - αδύνατον να εκχωρήσω %d ψηφιολέξεις\n"
ROFF_CMD_FROM_FILE_ERROR
	"Σφάλμα διερμηνείας *roff από το αρχείο %s\n"
MANROFFSEQ_ERROR
	"Σφάλμα στο MANROFFSEQ.  Χρήση της προεπιλογής συστήματος.\n"
ROFF_CMD_FROM_COMMANDLINE_ERROR
	"Σφάλμα διερμηνείας *roff από τη γραμμή διαταγών.\n"
UNRECOGNIZED_LINE
	"Η γραμμή %s στο αρχείο διευθέτησης δεν αναγνωρίστηκε (αγνοήθηκε).\n"
GETVAL_ERROR
	"man-config.c: εσωτερικό σφάλμα: η συμβολοσειρά %s δεν βρέθηκε\n"
FOUND_MANDIR
	"βρέθηκε ο κατάλογος man  %s\n"
FOUND_MAP
	"βρέθηκε η αντιστοιχία %s --> %s\n"
FOUND_CATDIR
	"ο αντίστοιχος κατάλογος cat είναι %s\n"
LINE_TOO_LONG
	"Γραμμή πολύ μεγάλη στο αρχείο διευθέτησης\n"
SECTION
	"\nενότητα: %s\n"
UNLINKED
	"αποσύνδεση %s\n"
GLOBBING
	"ταίριασμα ονόματος σελίδας με την έκφραση %s\n"
EXPANSION_FAILED
	"Η απόπειρα [%s] ταιριάσματος ονόματος ήταν χωρίς αποτέλεσμα\n"
OPEN_ERROR
	"Δεν μπορώ ν' ανοίξω τη σελίδα εγχειριδίου %s\n"
READ_ERROR
	"Σφάλμα κατα την ανάγνωση της σελίδας εγχειριδίου %s\n"
FOUND_EQN
	"βρέθηκε ο προεπεξεργαστής eqn(1)\n"
FOUND_GRAP
	"βρέθηκε ο προεπεξεργαστής grap(1)\n"
FOUND_PIC
	"βρέθηκε ο προεπεξεργαστής pic(1)\n"
FOUND_TBL
	"βρέθηκε ο προεπεξεργαστής tbl(1)\n"
FOUND_VGRIND
	"βρέθηκε ο προεπεξεργαστής vgrind(1)\n"
FOUND_REFER
	"βρέθηκε ο προεπεξεργαστής refer(1)\n"
ROFF_FROM_COMMAND_LINE
	"ανάλυση οδηγιών από τη γραμμή διαταγών\n"
ROFF_FROM_FILE
	"ανάλυση οδηγιών από το αρχείο %s\n"
ROFF_FROM_ENV
	"ανάλυση οδηγιών από το περιβάλλον\n"
USING_DEFAULT
	"χρήση ακολουθίας προεπιλεγμένου προεπεξεργαστή\n"
PLEASE_WAIT
	"Μορφοποίηση σελίδας, παρακαλώ περιμένετε...\n"
CHANGED_MODE
	"αλλαγή κατάστασης από %s σε %o\n"
CAT_OPEN_ERROR
	"Αδύνατον το άνοιγμα του %s για γράψιμο.\n"
PROPOSED_CATFILE
	"θα προσπαθήσουμε να γράψουμε στο %s αν υπάρξει ανάγκη\n"
IS_NEWER_RESULT
	"κατάσταση του is_newer() = %d\n"
TRYING_SECTION
	"δοκιμή στην ενότητας %s\n"
SEARCHING
	"\nέρευνα στο %s\n"
ALREADY_IN_MANPATH
	"ναι αλλά το %s βρίσκεται ήδη στη διαδρομή\n"
CANNOT_STAT
	"Προειδοποίηση: αδύνατον να δοκιμάσω το αρχείο %s!\n"
IS_NO_DIR
	"Προειδοποίηση: το %s δεν είναι κατάλογος!\n"
ADDING_TO_MANPATH
	"προσθήκη του %s στη διαδρομή αναζήτησης\n"
PATH_DIR
	"\nκατάλογος διαδρομής %s "
IS_IN_CONFIG
	"βρίσκεται στο αρχείο διευθέτησης\n"
IS_NOT_IN_CONFIG
	"δεν βρίσκεται στο αρχείο διευθέτησης\n"
MAN_NEARBY
	"αλλά υπάρχει ένας κατάλογος man εδώ κοντά\n"
NO_MAN_NEARBY
	"και δεν βρίσκουμε κατάλογο man στα πέριξ\n"
ADDING_MANDIRS
	"\nπροσθήκη υποχρεωτικών καταλόγων man\n\n"
CATNAME_IS
	"στο convert_to_cat (), το cat_name είναι: %s\n"
NO_EXEC
	"\nδεν εκτελείται η εντολή:\n  %s\n"
USAGE1
	"σύνταξη: %s [-adfhktwW] [ενότητα] [-M διαδρομή] [-P σελιδοποιός] [-S λίστα]\n\t"
USAGE2
	"[-m σύστημα] "
USAGE3
	"[-p συμβολοσειρά] όνομα ...\n\n"
USAGE4
	"  a : να βρεθούν όλα τα λήμματα που ταιριάζουν\n\
  c : μη χρήση του αρχείου cat\n\
  d : εμφανίζει πληροφορίες αποσφαλμάτωσης\n\
  D : όπως η -d, αλλά επιπλέον εμφάνιση των σελίδων\n\
  f : ισοδύναμη με whatis(1)\n\
  h : εμφάνιση αυτού εδώ του μηνύματος βοήθειας\n\
  k : ισοδύναμη με apropos(1)\n\
  K : έρευνα για τη συμβολοσειρά σ' όλες τις σελίδες\n"
USAGE5
	"  t : χρήση troff για τη μορφοποίηση σελίδων προς εμφάνιση\n"
USAGE6
	"\
  w : υποδεικνύει την τοποθεσία των σελίδων που θα εμφανισθούν\n\
      (αν δεν δοθεί όνομα: εμφάνιση των καταλόγων που θα ερευνηθούν)\n\
  W : όπως η -w, αλλά εμφάνιση μόνο των ονομάτων των αρχείων\n\n\
  C αρχείο   : χρησιμοποιεί το «αρχείο» ως αρχείο διευθέτησης\n\
  M διαδρομή   : καθορίζει τη διαδρομή αναζήτησης σελίδων σε «διαδρομή»\n\
  P σελιδοποιός  : χρήση του προγράμματος «σελιδοποιός» για την εμφάνιση των σελίδων\n\
  S λίστα   : λίστα ενοτήτων που χωρίζονται με άνω και κάτω τελείες\n"
USAGE7
	"  m σύστημα : αναζήτηση για εναλλακτικές σελίδες εγχειριδίου στο σύστημα\n"
USAGE8
	"  p συμβολοσειρά : η συμβολοσειρά καθορίζει ποιός προεπεξεργαστής να τρέξει\n\
               e - [n]eqn(1)   p - pic(1)    t - tbl(1)\n\
               g - grap(1)     r - refer(1)  v - vgrind(1)\n"
USER_CANNOT_OPEN_CAT
	"ακόμη και ο πραγματικός χρήστης δεν μπορεί ν' ανοίξει το αρχείο cat\n"
USER_CAN_OPEN_CAT
	"αλλά ο πραγματικός χρήστης μπορεί ν' ανοίξει το αρχείο cat\n"
CANNOT_FORK
	"σφάλμα κατά την κλωνοποίηση της εντολής _%s_\n"
WAIT_FAILED
	"σφάλμα κατά την αναμονή του παιδιού _%s_\n"
GOT_WRONG_PID
	"πολύ περίεργο ..., έλαβα λάθος pid ενώ ανέμενα το παιδί μου\n"
CHILD_TERMINATED_ABNORMALLY
	"μοιραίο σφάλμα : η εντολή _%s_ δεν τερματίστηκε ομαλά\n"
IDENTICAL
	"Η σελίδα εγχειριδίου %s είναι πανομοιότυπη με την %s\n"
MAN_FOUND
	"Βρήκα την(τις) σελίδα(ες):\n"
NO_TROFF
	"σφάλμα: δεν ορίστηκε εντολή TROFF στο %s\n"
NO_CAT_FOR_NONSTD_LL
	"ουδεμία σελίδα cat αποθηκεύτηκε εξ αιτίας μη τυποποιημένου μήκους γραμμής\n"
